@@ -342,9 +342,7 @@ AWT_ASSERT_APPKIT_THREAD;
 
 #define DRAGMASK (NSMouseMovedMask | NSLeftMouseDraggedMask | NSRightMouseDownMask | NSRightMouseDraggedMask | NSLeftMouseUpMask | NSRightMouseUpMask | NSFlagsChangedMask | NSKeyDownMask)
 
-#if defined(MAC_OS_X_VERSION_10_12) && \
-   MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_12 && \
-   __LP64__
+#if defined(MAC_OS_X_VERSION_10_12) && __LP64__
    // 10.12 changed `mask` to NSEventMask (unsigned long long) for x86_64 builds.
 - (NSEvent *)nextEventMatchingMask:(NSEventMask)mask
 #else
