@@ -236,7 +236,7 @@ public class TrueTypeFont extends FileFont {
             return useNatives;
         }
         if (!FontUtilities.isSolaris || useJavaRasterizer ||
-            FontUtilities.useT2K || nativeNames == null ||
+            FontUtilities.useJDKScaler || nativeNames == null ||
             getDirectoryEntry(EBLCTag) != null ||
             GraphicsEnvironment.isHeadless()) {
             checkedNatives = true;
@@ -896,15 +896,6 @@ public class TrueTypeFont extends FileFont {
         }
     }
 
-    @Override
-    protected long getLayoutTableCache() {
-        try {
-          return getScaler().getLayoutTableCache();
-        } catch(FontScalerException fe) {
-            return 0L;
-        }
-    }
-
     @Override
     protected byte[] getTableBytes(int tag) {
         ByteBuffer buffer = getTableBuffer(tag);
