@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2018, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -913,13 +913,6 @@ public abstract class WComponentPeer extends WObjectPeer
     public native void beginValidate();
     public native void endValidate();
 
-    /**
-     * DEPRECATED
-     */
-    public Dimension preferredSize() {
-        return getPreferredSize();
-    }
-
     /**
      * register a DropTarget with this native peer
      */
