@@ -30,7 +30,9 @@ import java.io.IOException;
 import java.text.BreakIterator;
 import java.util.Collections;
 import java.util.HashSet;
+import java.util.Map;
 import java.util.Set;
+import java.util.WeakHashMap;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -180,6 +182,8 @@ public class JavacTrees extends DocTrees {
     private ParserFactory parser;
     private Symtab syms;
 
+    private final Map<Type, Type> extraType2OriginalMap = new WeakHashMap<>();
+
     // called reflectively from Trees.instance(CompilationTask task)
     public static JavacTrees instance(JavaCompiler.CompilationTask task) {
         if (!(task instanceof BasicJavacTask))
@@ -439,7 +443,11 @@ public class JavacTrees extends DocTrees {
                 // we first check if qualifierExpression identifies a type,
                 // and if not, then we check to see if it identifies a package.
                 Type t = attr.attribType(ref.qualifierExpression, env);
-                if (t.isErroneous()) {
+
+                if (t.getKind() == TypeKind.ARRAY) {
+                    // cannot refer to an array type
+                    return null;
+                } else if (t.isErroneous()) {
                     JCCompilationUnit toplevel =
                         treeMaker.TopLevel(List.nil());
                     final ModuleSymbol msym = modules.getDefaultModule();
@@ -447,6 +455,9 @@ public class JavacTrees extends DocTrees {
                     toplevel.packge = msym.unnamedPackage;
                     Symbol sym = attr.attribIdent(ref.qualifierExpression, toplevel);
 
+                    if (sym == null)
+                        return null;
+
                     sym.complete();
 
                     if ((sym.kind == PCK || sym.kind == TYP) && sym.exists()) {
@@ -1083,6 +1094,20 @@ public class JavacTrees extends DocTrees {
         if (errorType instanceof com.sun.tools.javac.code.Type.ErrorType) {
             return ((com.sun.tools.javac.code.Type.ErrorType)errorType).getOriginalType();
         }
+        if (errorType instanceof com.sun.tools.javac.code.Type.ClassType &&
+            errorType.getKind() == TypeKind.ERROR) {
+            ClassType ct = (ClassType) errorType;
+            return extraType2OriginalMap.computeIfAbsent(ct, tt ->
+                    new ClassType(ct.getEnclosingType(), ct.typarams_field,
+                                  ct.tsym, ct.getMetadata()) {
+                        @Override
+                        public Type baseType() { return ct; }
+                        @Override
+                        public TypeKind getKind() {
+                            return TypeKind.DECLARED;
+                        }
+                    });
+        }
 
         return com.sun.tools.javac.code.Type.noType;
     }
@@ -1211,7 +1236,7 @@ public class JavacTrees extends DocTrees {
         jcCompilationUnit.lineMap = jcCompilationUnit.getLineMap();
         jcCompilationUnit.modle = psym.modle;
         jcCompilationUnit.sourcefile = jfo;
-        jcCompilationUnit.namedImportScope = new NamedImportScope(psym, jcCompilationUnit.toplevelScope);
+        jcCompilationUnit.namedImportScope = new NamedImportScope(psym);
         jcCompilationUnit.packge = psym;
         jcCompilationUnit.starImportScope = new StarImportScope(psym);
         jcCompilationUnit.toplevelScope = WriteableScope.create(psym);
