@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -69,9 +69,6 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
         // the environment -- set the layout manager, add buttons,
         // etc.
         this.setLayout (new BorderLayout ());
-        Sysout.createDialogWithInstructions(new String[]
-            {"This is an automatic test. Simply wait until it is done."
-            });
         toolkitClassName = Toolkit.getDefaultToolkit().getClass().getName();
     }
 
@@ -151,7 +148,7 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
 
         recreateGUI();
 
-        Sysout.println("Stage 1 in progress...");
+        System.out.println("Stage 1 in progress...");
 
         dialog.setModal(true);
         dialog.setAutoRequestFocus(false);
@@ -175,7 +172,7 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
 
         recreateGUI();
 
-        Sysout.println("Stage 2 in progress...");
+        System.out.println("Stage 2 in progress...");
 
         setVisible(focusedFrame, false);
 
@@ -283,7 +280,7 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
         // On Windows, an owned Window will not be focused on its showing
         // if the owner is not currently active.
         if ("sun.awt.windows.WToolkit".equals(toolkitClassName)) {
-            Sysout.println("Stage 5.1 - Skiping.");
+            System.out.println("Stage 5.1 - Skiping.");
         } else {
             setVisible(ownedWindow, true);
             setVisible(frame, false); // 'ownedWindow' will be shown along with the owner.
@@ -306,9 +303,9 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
         ///////////////////////////////////
 
         if ("sun.awt.motif.MToolkit".equals(toolkitClassName)) {
-            Sysout.println("Stage 6 - Skiping.");
+            System.out.println("Stage 6 - Skiping.");
         } else {
-            Sysout.println("Stage 6 in progress...");
+            System.out.println("Stage 6 in progress...");
 
             // ---
             // Testing the bug of activating invisible modal Dialog (awt_Window::SetAndActivateModalBlocker).
@@ -343,7 +340,7 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
             setVisible(dialog, false);
         }
 
-        Sysout.println("Test passed.");
+        System.out.println("Test passed.");
     }
 
     /*
@@ -356,7 +353,7 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
     void test(String msg, final Window showWindow, Window ownedWindow, final Button clickButton, boolean shouldFocusChange) {
         Window testWindow = (ownedWindow == null ? showWindow : ownedWindow);
 
-        Sysout.println(msg);
+        System.out.println(msg);
 
         if (showWindow.isVisible()) {
             showWindow.dispose();
@@ -428,7 +425,7 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
 
             if (!performed) {
                 // In case of loosing ACTION_PERFORMED, try once more.
-                Sysout.println("(ACTION_EVENT was not generated. One more attemp.)");
+                System.out.println("(ACTION_EVENT was not generated. One more attemp.)");
                 performed = Util.trackActionPerformed(clickButton, new Runnable() {
                         public void run() {
                             Util.clickOnComp(clickButton, robot);
@@ -470,140 +467,3 @@ class TestFailedException extends RuntimeException {
         super("Test failed: " + msg);
     }
 }
-
-/****************************************************
- Standard Test Machinery
- DO NOT modify anything below -- it's a standard
-  chunk of code whose purpose is to make user
-  interaction uniform, and thereby make it simpler
-  to read and understand someone else's test.
- ****************************************************/
-
-/**
- This is part of the standard test machinery.
- It creates a dialog (with the instructions), and is the interface
-  for sending text messages to the user.
- To print the instructions, send an array of strings to Sysout.createDialog
-  WithInstructions method.  Put one line of instructions per array entry.
- To display a message for the tester to see, simply call Sysout.println
-  with the string to be displayed.
- This mimics System.out.println but works within the test harness as well
-  as standalone.
- */
-
-class Sysout
-{
-    static TestDialog dialog;
-
-    public static void createDialogWithInstructions( String[] instructions )
-    {
-        dialog = new TestDialog( new Frame(), "Instructions" );
-        dialog.printInstructions( instructions );
-//        dialog.setVisible(true);
-        println( "Any messages for the tester will display here." );
-    }
-
-    public static void createDialog( )
-    {
-        dialog = new TestDialog( new Frame(), "Instructions" );
-        String[] defInstr = { "Instructions will appear here. ", "" } ;
-        dialog.printInstructions( defInstr );
-//        dialog.setVisible(true);
-        println( "Any messages for the tester will display here." );
-    }
-
-
-    public static void printInstructions( String[] instructions )
-    {
-        dialog.printInstructions( instructions );
-    }
-
-
-    public static void println( String messageIn )
-    {
-        dialog.displayMessage( messageIn );
-    }
-
-}// Sysout  class
-
-/**
-  This is part of the standard test machinery.  It provides a place for the
-   test instructions to be displayed, and a place for interactive messages
-   to the user to be displayed.
-  To have the test instructions displayed, see Sysout.
-  To have a message to the user be displayed, see Sysout.
-  Do not call anything in this dialog directly.
-  */
-class TestDialog extends Dialog
-{
-
-    TextArea instructionsText;
-    TextArea messageText;
-    int maxStringLength = 80;
-
-    //DO NOT call this directly, go through Sysout
-    public TestDialog( Frame frame, String name )
-    {
-        super( frame, name );
-        int scrollBoth = TextArea.SCROLLBARS_BOTH;
-        instructionsText = new TextArea( "", 15, maxStringLength, scrollBoth );
-        add( "North", instructionsText );
-
-        messageText = new TextArea( "", 5, maxStringLength, scrollBoth );
-        add("Center", messageText);
-
-        pack();
-
-//        setVisible(true);
-    }// TestDialog()
-
-    //DO NOT call this directly, go through Sysout
-    public void printInstructions( String[] instructions )
-    {
-        //Clear out any current instructions
-        instructionsText.setText( "" );
-
-        //Go down array of instruction strings
-
-        String printStr, remainingStr;
-        for( int i=0; i < instructions.length; i++ )
-        {
-            //chop up each into pieces maxSringLength long
-            remainingStr = instructions[ i ];
-            while( remainingStr.length() > 0 )
-            {
-                //if longer than max then chop off first max chars to print
-                if( remainingStr.length() >= maxStringLength )
-                {
-                    //Try to chop on a word boundary
-                    int posOfSpace = remainingStr.
-                        lastIndexOf( ' ', maxStringLength - 1 );
-
-                    if( posOfSpace <= 0 ) posOfSpace = maxStringLength - 1;
-
-                    printStr = remainingStr.substring( 0, posOfSpace + 1 );
-                    remainingStr = remainingStr.substring( posOfSpace + 1 );
-                }
-                //else just print
-                else
-                {
-                    printStr = remainingStr;
-                    remainingStr = "";
-                }
-
-                instructionsText.append( printStr + "\n" );
-
-            }// while
-
-        }// for
-
-    }//printInstructions()
-
-    //DO NOT call this directly, go through Sysout
-    public void displayMessage( String messageIn )
-    {
-        messageText.append( messageIn + "\n" );
-        System.out.println(messageIn);
-    }
-
-}// TestDialog  class
