@@ -23,7 +23,7 @@
 
 /*
  * @test
- * @bug 8144095 8164825 8169818 8153402 8165405 8177079 8178013 8167554
+ * @bug 8144095 8164825 8169818 8153402 8165405 8177079 8178013 8167554 8166232
  * @summary Test Command Completion
  * @modules jdk.compiler/com.sun.tools.javac.api
  *          jdk.compiler/com.sun.tools.javac.main
@@ -123,6 +123,14 @@ public class CommandCompletionTest extends ReplToolTesting {
         );
     }
 
+    @Test
+    public void testHistory() {
+        test(false, new String[] {"--no-startup"},
+                a -> assertCompletion(a, "/hi|", false, "/history "),
+                a -> assertCompletion(a, "/history |", false, "-all")
+        );
+    }
+
     @Test
     public void testDrop() {
         test(false, new String[] {"--no-startup"},
